# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
import urllib
import os
from threading import Thread
from gi.repository import GObject
from gi.repository import GdkPixbuf
import time
import md5

class CoverImage():
	def __init__(self, remote_get_cover, builder, settings, update_mpris):
        self.update_mpris = update_mpris
        self.remote_get_cover = remote_get_cover
        self.image = builder.get_object("albumcoverimage")
        self.spinner = builder.get_object("albumcoverspinner")
        self.box = builder.get_object("albumcoverbox")
        self.separator = builder.get_object("coverseparator")
        self.settings = settings
        self.pixbuf = None
        self.pid = 0
        self.loading = 0
        self.disabled = 0

    def update(self):
        ok = self.settings.get("album-covers")
        if ok:
            if self.disabled:
                self.box.show()
                self.separator.show()
                self.disabled = 0
            if not self.loading:
                self.image.hide()
                self.spinner.start()
                self.spinner.show()
                self.loading = 1
            self.pid += 1
            pid = self.pid
            self.update_mpris(False)
            downloader = CoverImageDownloader(self.remote_get_cover, self.set_cover, self.update_mpris_cover, pid, self.settings.get("mpris-album-covers"))
            downloader.start()
        elif not self.disabled:
            self.disabled = 1
            self.box.hide()
            self.separator.hide()

    def set_cover(self, image, pid):
        if pid == self.pid and not self.disabled:
            if not image:
                self.loading = 0
                self.image.hide()
                self.spinner.hide()
                self.pixbuf = None
                self.spinner.stop()
            else:
                self.loading = 0
                self.pixbuf = image
                self.image.set_from_pixbuf(image)
                self.spinner.hide()
                self.spinner.stop()
                self.image.show()

    def update_mpris_cover(self, pid, path):
        if pid == self.pid:
            if path:
                self.update_mpris(path)
            else:
                self.update_mpris(False)
        

class CoverImageDownloader(Thread):
	def __init__(self, remote_get_cover, callback1, callback2, pid, mpris):
        self.remote_get_cover = remote_get_cover
        self.callback1 = callback1
        self.callback2 = callback2
        self.mpris = mpris
        self.pid = pid
        Thread.__init__(self)

    def run(self):
        data = self.remote_get_cover()
        if data:
            m = md5.new()
            m.update(data)
            md5sum = m.hexdigest()
            if md5sum != "3580cc6daa5e7b45b4e2926dfa1a820e":
                try:
                    loader = GdkPixbuf.PixbufLoader()
				    loader.write(data)
				    loader.close()
                    pixbuf = loader.get_pixbuf()
                    npixbuf = pixbuf.scale_simple(66, 66, GdkPixbuf.InterpType.BILINEAR)
                except Exception as e:
                    print "Error", e
                else:
                    GObject.idle_add(self.callback1, npixbuf, self.pid)
                    if self.mpris:
                        try:
                            npixbuf = pixbuf.scale_simple(60, 60, GdkPixbuf.InterpType.BILINEAR)
                            npixbuf.savev("/tmp/" + md5sum + ".jpg", "jpeg", "", "")
                        except Exception as e:
                            print "Error", e
                        else:
                            GObject.idle_add(self.callback2, self.pid, "/tmp/" + md5sum + ".jpg")
                    return
        GObject.idle_add(self.callback1, False, self.pid)
        GObject.idle_add(self.callback2, self.pid, False)
            
