# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# This file is in the public domain
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('dockremoteclient')
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import Gtk # pylint: disable=E0611
from gi.repository import Gio # pylint: disable=E0611
from gi.repository import GObject
import logging
logger = logging.getLogger('dockremoteclient')
import time
import os

from dockremoteclient_lib import Window
from dockremoteclient.AboutDockRemoteClientDialog import AboutDockRemoteClientDialog
from dockremoteclient.PreferencesDockRemoteClientDialog import PreferencesDockRemoteClientDialog
from dockremoteclient.SyncprogressDialog import SyncprogressDialog
from dockremoteclient.ConnectionerrorDialog import ConnectionerrorDialog
from dockremoteclient.FirstrunDialog import FirstrunDialog
from dockremoteclient.LyricspanelDialog import LyricspanelDialog
from dockremoteclient.Player import Player
from dockremoteclient.Library import Library
from dockremoteclient.Remote import Remote
from dockremoteclient.Timer import Timer
from dockremoteclient.Queue import Queue
from dockremoteclient.Settings import Settings
from dockremoteclient.MusicData import MusicData
from dockremoteclient.MediaKeyHandler import MediaKeyHandler
from dockremoteclient.Notify import DockRemoteClientNotifyOsd
from dockremoteclient.SoundMenu import SoundMenuControls

def tra(t):
       return t

_ = tra

# See dockremoteclient_lib.Window.py for more details about how this class works
class DockRemoteClientWindow(Window):
    __gtype_name__ = "DockRemoteClientWindow"
   
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(DockRemoteClientWindow, self).finish_initializing(builder)
        self.PreferencesDialog = PreferencesDockRemoteClientDialog
        self.settings = Settings(self.on_setting_changed)
        self.stopped_loading = 0
        if self.settings.get("ip-address") == "":
            self.firstrundialog = FirstrunDialog()
            self.firstrundialog.set_settings(self.settings, self.on_stop_loading)
            self.response = self.firstrundialog.run()
            if self.response == -1:
                self.stopped_loading = 1
                return False
        self.database = MusicData(os.getenv("HOME") + "/" + self.settings.get("db-path"))
        self.database.connect()
        self.player = Player(self)
        self.library = Library(self.builder, self.play_song, self.database)
        self.AboutDialog = AboutDockRemoteClientDialog
        self.syncprogressdialog = SyncprogressDialog()
        self.remote = Remote(self.settings.get("ip-address"), self.player.handlers, self.on_connection_error)
        self.lyricspaneldialog = LyricspanelDialog()
        self.lyricsmenuitem = self.builder.get_object("lyricsmenuitem")
        self.lyricspaneldialog.import_searchers(self.lyricsmenuitem, self.on_lyricsmenuitem_clicked, self.remote)
        self.syncprogressdialog.give_parent(self)
        self.settings.activate()
        self.notify = DockRemoteClientNotifyOsd(self.remote, self.settings)
        self.timer = Timer(self.update_fraction, self.remote.update, self.prepare_next_song)
        self.lastvolumeupdate = 0

        self.deletefromqueuebutton = self.builder.get_object("deletefromqueuebutton")
        self.upinqueuebutton = self.builder.get_object("upinqueuebutton")
        self.downinqueuebutton = self.builder.get_object("downinqueuebutton")
        self.titlelabel = self.builder.get_object("titlelabel")
        self.artistlabel = self.builder.get_object("artistlabel")
        self.albumlabel = self.builder.get_object("albumlabel")
        self.playbutton = self.builder.get_object("playbutton")
        self.prefbutton = self.builder.get_object("prefbutton")
        self.syncbutton = self.builder.get_object("syncbutton")
        self.volumebutton = self.builder.get_object("volumebutton")
        self.fractionbar = self.builder.get_object("fractionbar")
        self.fraction = self.builder.get_object("fraction")
        self.timepassed = self.builder.get_object("timepassed")
        self.shufflecombo = self.builder.get_object("shufflecombo")
        self.repeatcombo = self.builder.get_object("repeatcombo")
        self.queuebox = self.builder.get_object("queuebox")
        self.showqueuebutton = self.builder.get_object("showqueuebutton")
        self.queue = Queue(self.builder, self.remote, self.library, self.database)
        self.library.give_queue(self.queue, self.player)
        self.timer.start()
        DBusGMainLoop(set_as_default=True)
##### gsettings set com.canonical.indicator.sound interested-media-players "['vlc']"######
        self.soundmenu = SoundMenuControls('dockremoteclient', self)
        self.player.queueing = 0
        self.toggling = 0
        if self.settings.settings.get_boolean("media-keys"):
            self.mediakey = MediaKeyHandler(self.remote)
        self.player.set_parent(self)
        self.firstchanged = 1
        self.connect_pref_widgets()
        # Code for other initialization actions should be added here.
        
        self.gsettings = Gio.Settings("net.launchpad.dockremoteclient")
        self.gsettings.bind("show-queue", self.showqueuebutton, "active", Gio.SettingsBindFlags.DEFAULT)
        self.on_showqueuebutton_toggled(self.showqueuebutton)
        self.sliding = 0

        GObject.idle_add(self.remote.update)
        

        
    
        
    def set_shuffle(self, widget, v):
        pass        
    def set_repeat(self, widget, v):
        pass
    def on_connection_error(self, v):
        self.timer.pause(True)
        self.connectionerrordialog = ConnectionerrorDialog()
        self.connectionerrordialog.set_settings(self.settings, self.destroy, self.remote.update)
        self.connectionerrordialog.set_info(str(v), self.settings.get("ip-address"))
        self.connectionerrordialog.run()

    def on_lyricsmenuitem_clicked(self, widget, data=0):
        self.lyricspaneldialog.set_lyrics(self.remote.info["title"], self.remote.info["artist"], data)
        self.lyricspaneldialog.run()

    def on_stop_loading(self):
        self.stopped_loading = 1

    def on_playbutton_clicked(self, widget):
        logger.debug("Play/pause button clicked, calling player.playpause")
        self.player.playpause("playbutton")

    def on_showqueuebutton_toggled(self, widget):
        if widget.get_active():
            self.queuebox.show()
        else:
            self.queuebox.hide()

    def on_syncbutton1_activate(self, widget):
        self.on_syncbutton_activate(widget)

    def on_syncbutton_activate(self, widget):
        logger.debug("Sync button clicked, showing dialog")
        self.syncprogressdialog.reset()
        self.syncprogressdialog.show()
        self.syncprogressdialog.start()

    def on_volumebutton_value_changed(self, widget, value):
        if (time.time() - self.lastvolumeupdate) > 0.2:
            logger.debug("Setting volume to %s" % value)
            self.remote.set_volume(value)
            self.lastvolumeupdate = time.time()

    def on_volumebutton_released(self, widget):
        self.on_volumebutton_value_changed(self, self.volumebutton.get_value())

    def on_volumebutton_leave_notify_event(self, widget, event):
        self.on_volumebutton_value_changed(self, self.volumebutton.get_value())
        
    def on_previoussong_clicked(self, widget):
        logger.debug("Previous button clicked, calling player.prev")
        self.player.prev()
     
    def on_nextsong_clicked(self, widget):
        logger.debug("Next button clicked, calling player.next")
        self.player.next()

    def update_fraction(self, val, pas, fut):
        #logger.debug("Updating fraction to %s" % val)
        if self.sliding < 2:
            self.fraction.set_value(float(val))
        self.timepassed.set_text(str(int(pas)/60).zfill(2) + ":" + str(int(pas)%60).zfill(2) + _(" of ") + str(int(fut)/60).zfill(2) + ":" + str(int(fut)%60).zfill(2))

    def on_updated_time(self, val):
        #logger.debug("Fraction changed from iDevice: updating gui")
        self.timer.update(self.remote.info["fraction"], self.remote.info["time"], self.remote.info["duration"], self.remote.info["state"])

    def prepare_next_song(self):
        if self.queue.queue_isnt_empty():
            logger.debug("Playing first song from the queue")
            self.queue.play_first_song()

    def play_song(self, sid):
        logger.debug("Playing song with sid=%s" % sid)
        self.remote.play_song(sid)

    def on_volume_changed(self, value):
        logger.debug("Volume changed from iDevice: updating gui (%s)" % value)
        self.volumebutton.set_value(float(value))

    def on_shuffle_changed(self, value):
        logger.debug("Shuffle mode changed from iDevice: updating gui (%s)" % value)
        if self.toggling:
            self.toggling = 0
        elif value == "on":
            self.shufflecombo.set_active(0)
        else:
            self.shufflecombo.set_active(1)

    def on_repeat_changed(self, value):
        v = self.repeatcombo.get_active()
        if self.toggling:
            self.toggling = 0
        elif value == "all" and 1 != v:
            self.repeatcombo.set_active(1)
        elif value == "one" and 2 != v:
            self.repeatcombo.set_active(2)
        elif value == "none" and 0 != v:
            self.repeatcombo.set_active(0)

    def on_shufflecombo_changed(self, widget):
        if self.firstchanged == 1:
            self.firstchanged = 0
            return
        v = widget.get_active()
        if v == 0:
            if self.remote.info["shuffle"] != "on":
                self.toggling = 1
                self.remote.toggle_shuffle()
            else:
                self.on_shuffle_changed("on")
        elif v == 1:
            
            if self.remote.info["shuffle"] == "on":
                self.toggling = 1
                self.remote.toggle_shuffle()
            else:
                self.on_shuffle_changed("no")

    def on_repeatcombo_changed(self, widget):
        v = widget.get_active()
        if v == 0:
            if self.remote.info["repeat"] == "all":
                self.toggling = 1
                self.remote.toggle_repeat()
            if self.remote.info["repeat"] == "all" or self.remote.info["repeat"] == "one":
                self.toggling = 1
                self.remote.toggle_repeat()
        elif v == 1:
            if self.remote.info["repeat"] == "one":
                self.toggling = 1
                self.remote.toggle_repeat()
            if self.remote.info["repeat"] == "one" or self.remote.info["repeat"] == "none":
                self.toggling = 1
                self.remote.toggle_repeat()
        elif v == 2:
            if self.remote.info["repeat"] == "none":
                self.toggling = 1
                self.remote.toggle_repeat()
            if self.remote.info["repeat"] == "none" or self.remote.info["repeat"] == "all":
                self.toggling = 1
                self.remote.toggle_repeat()

    def on_setting_changed(self, settings, key, data=None):
        if key == "ip-address":
            self.remote.update_ip_address(settings.get_string(key))

    def connect_pref_widgets(self):
        self.settings.connect_widget(self.repeatcombo, "repeat-mode", "active")
        #self.shufflecombo.set_active(self.settings.get("shuffle-mode"))
        self.settings.connect_widget(self.shufflecombo, "shuffle-mode", "active")
   
    def on_fractionbar_button_release_event(self, widget, event):
        if event.button == 1:
            self.remote.set_fraction(self.fraction.get_value())
            self.sliding = 0
  
    def on_fractionbar_change_value(self, widget, event, value):
        self.sliding = self.sliding + 1
