# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# This file is in the public domain
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from dockremoteclient_lib.helpers import get_builder

import gettext
from threading import Thread
from gi.repository import GObject
import urllib
from gettext import gettext as _
gettext.textdomain('dockremoteclient')

class FirstrunDialog(Gtk.Dialog):
    __gtype_name__ = "FirstrunDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated FirstrunDialog object.
        """
        builder = get_builder('FirstrunDialog')
        new_object = builder.get_object('firstrun_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a FirstrunDialog object with it in order to
        finish initializing the start of the new FirstrunDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        self.ipaddressentry = self.builder.get_object('ipaddressentry')
        self.spinner = self.builder.get_object('spinner1')
        self.okbutton = self.builder.get_object('okbutton')
        self.cancelbutton = self.builder.get_object('cancelbutton')
        self.errorlabel = self.builder.get_object('errorlabel')
        self.verifyinglabel = self.builder.get_object('verifyinglabel')
        self.stop_emit = 1

    def on_firstrun_dialog_response(self, dialog, response):
        if self.stop_emit:
            self.emit_stop_by_name('response')
            return False

    def set_settings(self, settings, stop):
        self.settings = settings
        self.on_stop_loading = stop

    def on_firstrun_dialog_delete_event(self, widget, event):
        if self.stop_emit:
            return True

    def on_okbutton_clicked(self, widget, data=None):
        ip = self.ipaddressentry.get_text()
        self.tester = UrlTester(self.spinner, self.on_url_tested, ip)
        self.okbutton.hide()
        self.spinner.show()
        self.verifyinglabel.show()
        self.errorlabel.hide()
        self.tester.start()

    def on_firstrun_dialog_close(self, widget):
        self.destroy()

    def on_cancelbutton_clicked(self, widget):
        if self.spinner.get_visible():
            self.tester.cancel()
            self.okbutton.show()
            self.spinner.hide()
            self.verifyinglabel.hide()
        else:
            self.destroy()
            self.on_stop_loading()
            

    def on_url_tested(self, success, error):
        self.okbutton.show()
        self.spinner.hide()
        self.verifyinglabel.hide()
        if success:
            self.settings.set("ip-address", error)
            self.stop_emit = 0
            self.response(1)
            self.destroy()
        else:
            self.errorlabel.set_text(_("Error: ") + str(error) + _("\nCheck that the IP address is correct and that the device is on and connected to the same network as this pc and that the app is opened."))
            self.errorlabel.show()
            

class UrlTester(Thread):
    def __init__(self, spinner, callback, url):
        if url[:7] == "http://":
            urlstart = 7
        elif url[:1] == "/":
            urlstart = 1
        else:
            urlstart = 0
        url = url[urlstart:]
        if url[-2:] == "//":
            endurl = -2
        elif url[-1:] == "/":
            endurl = -1
        else:
            endurl = len(url)
        url = url[:endurl]
		self.url = "http://" + url + "/"
        self.callback = callback
        self.spinner = spinner
        self.active = 1
        Thread.__init__(self)
        

    def run(self):
        GObject.idle_add(self.spinner.start)
        try:
		    stuff = urllib.urlopen(self.url).read();
        except Exception as e:
            print self.url
            print "Error :", e
            if self.active:
                GObject.idle_add(self.spinner.stop)
                GObject.idle_add(self.callback, False, e)
        else:
            if self.active:
                GObject.idle_add(self.spinner.stop)
                GObject.idle_add(self.callback, True, self.url)
    def cancel(self):
        self.active = 0


if __name__ == "__main__":
    dialog = FirstrunDialog()
    dialog.show()
    Gtk.main()
