# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
from gi.repository import Gtk
from gi.repository import Gdk

import gettext
from gettext import gettext as _
gettext.textdomain('dockremoteclient')


class Library():
    TARGETS = [
    ('MY_TREE_MODEL_ROW', Gtk.TargetFlags.SAME_WIDGET, 0),
    ('text/plain', 0, 1),
    ('TEXT', 0, 2),
    ('STRING', 0, 3),
    ]

	def __init__(self, builder, handle_song_clicked, db):
        self.queue = None
        self.player = None
        self.database = db
		self.treeview = builder.get_object("library")
        self.librarycontextmenu = builder.get_object("librarycontextmenu")
		self.model = Gtk.ListStore(str, str, str, str, str, str) ## title, artist, album ,sid, aid, alid


		cell0 = Gtk.CellRendererText ()
		self.column_title = Gtk.TreeViewColumn(_("Title"), cell0, text=0)
		self.column_title.set_resizable(True)
		self.column_title.set_expand(True)
        self.column_title.set_sort_column_id(0)
        self.column_title.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.column_title.set_resizable(True)
        self.column_title.set_fixed_width(430)
		self.treeview.append_column (self.column_title)

        self.column_artist = Gtk.TreeViewColumn(_("Artist"), cell0, text=1)
		self.column_artist.set_resizable(True)
		self.column_artist.set_expand(True)
        self.column_artist.set_sort_column_id(1)
        self.column_artist.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.column_artist.set_resizable(True)
        self.column_artist.set_fixed_width(300)
		self.treeview.append_column (self.column_artist)

        self.column_album = Gtk.TreeViewColumn(_("Album"), cell0, text=2)
		self.column_album.set_resizable(True)
		self.column_album.set_expand(True)
        self.column_album.set_sort_column_id(2)
        self.column_album.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.column_album.set_resizable(True)
        self.column_album.set_fixed_width(300)
		self.treeview.append_column (self.column_album)

		self.model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
		self.model.set_sort_func(0, self.sort, 0)
		self.treeview.set_model(self.model)
        self.treeview.connect("row-activated", self.on_row_clicked)
        self.treeview.connect("key-press-event", self.on_key_pressed)
        self.handle_song_clicked = handle_song_clicked

        self.targets = [("song", 0, 0)]
        self.treeview.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.TARGETS, Gdk.DragAction.DEFAULT | Gdk.DragAction.MOVE | Gdk.DragAction.COPY)
        self.treeview.drag_source_add_text_targets()
        self.treeview.connect("drag_data_get", self.drag_data_get_data)
        self.selection = self.treeview.get_selection()
        self.selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        self.libraryframe = builder.get_object("libraryframe")
        self.libraryscrolledwindow = builder.get_object("libraryscrolledwindow")

        data = self.database.get_library()
        if len(data) == 0:
            self.libraryframe.show()
            self.libraryscrolledwindow.hide()
        else:
            self.libraryframe.hide()
            self.libraryscrolledwindow.show()
        for i in data:
            self.model.append(i)
        self.treeview.connect("button-press-event", self.on_library_button_press_event)
        self.treeview.connect("button-release-event", self.on_library_button_release_event)
        self.addsongtoqueueitem = builder.get_object("addsongtoqueueitem")
        self.addsongtoqueueitem.connect("activate", self.on_addsongtoqueueitem_activate)
        self.addartisttoqueueitem = builder.get_object("addartisttoqueueitem")
        self.addartisttoqueueitem.connect("activate", self.on_addartisttoqueueitem_activate)
        self.addalbumtoqueueitem = builder.get_object("addalbumtoqueueitem")
        self.addalbumtoqueueitem.connect("activate", self.on_addalbumtoqueueitem_activate)
        self.playsongitem = builder.get_object("playsongitem")
        self.playsongitem.connect("activate", self.on_playsongitem_activate)
        self.playartistitem = builder.get_object("playartistitem")
        self.playartistitem.connect("activate", self.on_playartistitem_activate)
        self.playalbumitem = builder.get_object("playalbumitem")
        self.playalbumitem.connect("activate", self.on_playalbumitem_activate)
        self.deleteitem = builder.get_object("deleteitem")
        self.deleteitem.connect("activate", self.on_deleteitem_activate)
        self.selection.connect("changed", self.on_selection_changed)
        self.controlpressed = 0
        self.shiftpressed = 0
        self.defer_select = False
    def on_selection_changed(self, selection):
        rows = selection.count_selected_rows()
        if rows > 1:
            self.addartisttoqueueitem.set_sensitive(False)
            self.addalbumtoqueueitem.set_sensitive(False)
            self.playartistitem.set_sensitive(False)
            self.playalbumitem.set_sensitive(False)
            self.playsongitem.set_sensitive(False)
        elif rows == 1:
            self.addartisttoqueueitem.set_sensitive(True)
            self.addalbumtoqueueitem.set_sensitive(True)
            self.playartistitem.set_sensitive(True)
            self.playalbumitem.set_sensitive(True)
            self.playsongitem.set_sensitive(True)          

    def give_queue(self, q, p):
        self.queue = q
        self.player = p

    def on_key_pressed(self, widget, event):
        if Gdk.keyval_name(event.keyval) == "Delete":
            self.on_deleteitem_activate(1)
        elif Gdk.keyval_name(event.keyval)[:7] == "Control":
            self.controlpressed = 1
        elif Gdk.keyval_name(event.keyval)[:5] == "Shift":
            self.shiftpressed = 1

    def on_key_released(self, widget, event):
        if Gdk.keyval_name(event.keyval)[:7] == "Control":
            self.controlpressed = 0
        elif Gdk.keyval_name(event.keyval)[:5] == "Shift":
            self.shiftpressed = 0

    def on_library_button_release_event(self, treeview, event):
	    self.selection.set_select_function(self.returnme, True)
        target = self.treeview.get_path_at_pos(int(event.x), int(event.y))	
        if (self.defer_select and target and self.defer_select == target[0] and not (event.x==0 and event.y==0)): # certain drag and drop
            self.treeview.set_cursor(target[0], target[1], False)
        self.defer_select=False


    def on_library_button_press_event(self, treeview, event):
        if event.button == 3 and self.selection.count_selected_rows():
            self.librarycontextmenu.popup( None, None, None, None, event.button, event.time)
            return True
        target = self.treeview.get_path_at_pos(int(event.x), int(event.y))
        if (target and event.type == Gdk.EventType.BUTTON_PRESS and not (event.state & (Gdk.ModifierType.CONTROL_MASK|Gdk.ModifierType.SHIFT_MASK)) and self.selection.path_is_selected(target[0])):
            self.selection.set_select_function(self.returnme, False)
            self.defer_select = target[0]

    def returnme(self, selection, model, path, boll, data):
        return data

    def get_selected_iters(self):
        if self.selection.count_selected_rows():
            pat = self.selection.get_selected_rows()
            iters = []
            for i in pat[1]:
                iters.append(self.model.get_iter(i))
            return iters

    def drag_data_get_data(self, treeview, context, selection, target_id, etime):
        model, pat = self.selection.get_selected_rows()
        iters = []
        data = []
        for i in pat:
            iters.append(self.model.get_iter(i))
        for iter in iters:
            dat = [model.get_value(iter, 0), model.get_value(iter, 1), model.get_value(iter, 2), model.get_value(iter, 3), model.get_value(iter, 4), model.get_value(iter, 5)]
            data.append("||".join(dat))
        selection.set(selection.get_target(), 8, "|**|".join(data))


    def on_row_clicked(self, treeview, path, col):
        ite = self.model.get_iter(path)
        sid = self.model.get_value(ite, 3)
        self.handle_song_clicked(sid)

    def sort(self, model, iter1, iter2, col):
		val1 = self.model.get_value(iter1, col).lower()
		val2 = self.model.get_value(iter2, col).lower()
		lis = [val1, val2]
		lis.sort()
		if lis[0] == val1:
			return -1
		if lis[1] == val1:
			return 1


    def populate(self, data):
        self.model.append("a a a a a a".split())
        self.model.clear()  
        if len(data) == 0:
            self.libraryframe.show()
            self.libraryscrolledwindow.hide()
        else:
            self.libraryframe.hide()
            self.libraryscrolledwindow.show()
        for i in data:
            self.model.append(i)
        self.database.fill_library(data)

    def get_iters_from_album(self, alid):
        iterlist = []
        iter = self.model.get_iter_first()
        while True:
            if str(self.model.get_value(iter, 5)) == str(alid):
                iterlist.append(iter)
            
            iter = self.model.iter_next(iter)
            if iter is None:
                break
        return iterlist

    
    def get_iters_from_artist(self, aid):
        iterlist = []
        iter = self.model.get_iter_first()
        while True:
            if str(self.model.get_value(iter, 4)) == str(aid):
                iterlist.append(iter)
            
            iter = self.model.iter_next(iter)
            if iter is None:
                break
        return iterlist

    def get_songs_from_iters(self, iterlist):
        songlist = []
        for iter in iterlist:
            songlist.append([self.model.get_value(iter, 0), self.model.get_value(iter, 1), self.model.get_value(iter, 2), self.model.get_value(iter, 3), self.model.get_value(iter, 4), self.model.get_value(iter, 5)])
        return songlist
            
    def on_addalbumtoqueueitem_activate(self, menu):
        iter = self.get_selected_iters()[0]
        alid = self.model.get_value(iter, 5)
        songlist = self.get_songs_from_iters(self.get_iters_from_album(alid))
        self.queue.add_to_queue(songlist)     
       
    def on_addartisttoqueueitem_activate(self, menu):
        iter = self.get_selected_iters()[0]
        aid = self.model.get_value(iter, 4)
        songlist = self.get_songs_from_iters(self.get_iters_from_artist(aid))
        self.queue.add_to_queue(songlist)

    def on_addsongtoqueueitem_activate(self, menu):
        iters = self.get_selected_iters()
        for iter in iters:
            self.queue.add_to_queue([self.model.get_value(iter, 0), self.model.get_value(iter, 1), self.model.get_value(iter, 2), self.model.get_value(iter, 3), self.model.get_value(iter, 4), self.model.get_value(iter, 5)])

    def on_deleteitem_activate(self, menu):
        iters = self.get_selected_iters()
        for iter in iters:
            self.database.remove_from_library(str(self.model.get_value(iter, 3)))
            self.model.remove(iter)

    def on_playsongitem_activate(self, menu):
        iter = self.get_selected_iters()[0]
        sid = self.model.get_value(iter, 3)
        self.player.play_song(sid)

    def on_playartistitem_activate(self, menu):
        iter = self.get_selected_iters()[0]
        aid = self.model.get_value(iter, 4)
        sid = self.model.get_value(iter, 3)
        self.player.play_song_from_artist(sid, aid)

    def on_playalbumitem_activate(self, menu):
        iter = self.get_selected_iters()[0]
        alid = self.model.get_value(iter, 5)
        sid = self.model.get_value(iter, 3)
        self.player.play_song_from_album(sid, alid)

    
        
        
        
