# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# This file is in the public domain
### END LICENSE
import os
from gi.repository import Gtk # pylint: disable=E0611
import imp
from dockremoteclient_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('dockremoteclient')
import logging
logger = logging.getLogger('dockremoteclient')




class LyricspanelDialog(Gtk.Dialog):
    __gtype_name__ = "LyricspanelDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated LyricspanelDialog object.
        """
        builder = get_builder('LyricspanelDialog')
        new_object = builder.get_object('lyricspanel_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a LyricspanelDialog object with it in order to
        finish initializing the start of the new LyricspanelDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        self.titleentry = builder.get_object("titleentry")
        self.artistentry = builder.get_object("artistentry")
        self.lyricsview = builder.get_object("lyricsview")
        self.lyricsbuffer = builder.get_object("lyricsbuffer")
        self.lyricsfoundbox = builder.get_object("lyricsfoundbox")
        self.lyricsnotfoundbox = builder.get_object("lyricsnotfoundbox")
        self.lyricsbadbox = builder.get_object("lyricsbadbox")
        self.researchbutton = builder.get_object("researchbutton")
        self.messagelabel = builder.get_object("messagelabel")
        self.notfoundlabel = builder.get_object("notfoundlabel")
        self.resultsbox = builder.get_object("resultsbox")
        self.sourcelinkbutton = builder.get_object("sourcelinkbutton")
        self.updatesearchbutton = builder.get_object("updatesearchbutton")
        self.updatesongbutton = builder.get_object("updatesongbutton")
        self.sourcecombobox = builder.get_object("sourcecombobox")
        self.artistentry.connect("changed", self.on_titleentry_changed)
        self.titleentry.connect("changed", self.on_titleentry_changed)
        self.researchbutton.connect("clicked", self.get_url)
        self.key = ""
        self.lyricsurl = ""

    def import_searchers(self, menuitem, callback, remote):
        self.remote = remote
        folder = os.path.join(os.path.normpath(os.path.join(os.path.dirname(__file__), "..")), "lyrics_searchers")
        l = os.listdir(folder)
        f = []
        self.search_func = {}
        self.search_url = {}
        self.search_from_url = {}
        self.lyricsmenu = Gtk.Menu()
        self.sourceliststore = Gtk.ListStore(str, int)
        index = 0
        for filename in l:
            n = os.path.basename(filename)
            filename = os.path.join(folder, filename)
            if n[-3:] == '.py' and n[:-3] != "__init__":
                (path, name) = os.path.split(filename)
                (name, ext) = os.path.splitext(name)
                (file, filename, data) = imp.find_module(name, [path])
                error_string = "\nError importing module " + n[:-3] + ":\n"
                try:
                    m = imp.load_module(name, file, filename, data)
                    k = m.searcher_title
                except Exception as e:
                    logger.error(error_string + e.__str__())
                    continue
                try:
                    self.search_func[k] = m.get_lyrics
                    self.search_url[k] = m.searcher_url
                    self.search_from_url[k] = m.get_lyrics_from_url
                except Exception as e:
                    logger.error(error_string + e.__str__())
                    try:
                        del self.search_func[k]
                    except:
                        pass
                    try:
                        del self.search_url[k]
                    except:
                        pass
                    try:
                        del self.search_from_url[k]
                    except:
                        pass
                        
                    continue
                item = Gtk.MenuItem(k)
                item.connect("activate", callback, k)
                item.show()
                self.lyricsmenu.append(item)
                self.sourceliststore.append([k, index])
                index += 1
        menuitem.set_submenu(self.lyricsmenu)
        self.lyricsmenu.show()
        self.sourcecombobox.set_model(self.sourceliststore)
        cell = Gtk.CellRendererText()
        self.sourcecombobox.pack_start(cell, True)
        self.sourcecombobox.add_attribute(cell, 'text', 0)

    def on_btn_ok_clicked(self, widget, data=None):
        self.hide()

    def on_lyricspanel_dialog_delete_event(self, widget, event):
        self.updatesongbutton.hide()
        self.hide()
        return True

    def set_lyrics(self, title, artist, source=0, hideupdate=True):
        if hideupdate:
            self.updatesongbutton.hide()
        self.changing = 1
        self.title = title
        self.artist = artist
        if type(source) == int:
            key = self.search_func.keys()[source]
        elif type(source) == str:
            key = source
        else:
            key = self.search_func.keys()[0]
        self.key = key
        func = self.search_func[key]
        r = func(title, artist, True)
        self.lyricsurl = r[1]
        l = r[0]
        self.titleentry.set_text(title)
        self.artistentry.set_text(artist)
        self.sourcelinkbutton.set_uri(self.lyricsurl)
        self.sourcelinkbutton.set_label(key)
        self.set_title(_("Lyrics search for ") + title + _(" on ") + key)
        keys = self.sourceliststore
        for i in keys:
            if keys.get_value(i.iter, 0) == self.key:
                self.sourcecombobox.set_active_iter(i.iter)
                break
        if type(l) == str:
            self.lyricsurl = self.search_url[key]
            self.lyricsbuffer.set_text(l)
            self.lyricsbadbox.hide()
            self.lyricsnotfoundbox.hide()
            self.lyricsfoundbox.show()
        elif type(l) == list and len(l):
            if len(l) > 50:
                l = l[:50]
            self.messagelabel.set_text(_("Lyrics not found on ") + key + "\n" + _("Did you mean:"))
            self.resultslist = {}
            group = None
            for i in self.resultsbox.get_children():
                self.resultsbox.remove(i)
            for [name, url] in l:
                self.resultslist[name] = url
                button = Gtk.RadioButton.new_with_label_from_widget(group, name)
                try:
                    arttit = name.split(" - ")
                    button.connect("toggled", self.set_result, url, arttit[1], arttit[0])
                except Exception as e:
                    arttit = ["", ""]
                    button.connect("toggled", self.set_result, url, arttit[1], arttit[0])
                self.resultsbox.pack_start(button, False, True, 0)
                button.show()
                if group == None:
                    group = button
                    self.set_result("", url, arttit[1], arttit[0])
            self.lyricsbadbox.show()
            self.lyricsfoundbox.hide()
            self.lyricsnotfoundbox.hide()
        else:
            t = _("Lyrics not found on ") + key + "."
            self.notfoundlabel.set_text(t)
            self.lyricsbadbox.hide()
            self.lyricsfoundbox.hide()
            self.lyricsnotfoundbox.show()
        self.changing = 0
            

    def set_result(self, widget, data, title, artist):
        self.lyricsurl = data
        self.resulttitle = title
        self.resultartist = artist

    def get_url(self, widget, event=None):
        func = self.search_from_url[self.key]
        l = func(self.lyricsurl)
        self.lyricsbuffer.set_text(l)
        self.sourcelinkbutton.set_uri(self.lyricsurl)
        self.sourcelinkbutton.set_label(self.key)
        self.titleentry.set_text(self.resulttitle)
        self.artistentry.set_text(self.resultartist)
        self.artist = self.resultartist
        self.title = self.resulttitle
        self.lyricsbadbox.hide()
        self.lyricsnotfoundbox.hide()
        self.lyricsfoundbox.show()
        
    def on_updatesearchbutton_clicked(self, widget):
        self.set_lyrics(self.titleentry.get_text(), self.artistentry.get_text(), self.key, False)

    def on_titleentry_changed(self, widget, data=None):
        if self.titleentry.get_text() == self.title and self.artistentry.get_text() == self.artist:
            self.updatesearchbutton.hide()
        else:
            self.updatesearchbutton.show()

    def on_sourcecombobox_changed(self, widget):
        if not self.changing:
            self.set_lyrics(self.titleentry.get_text(), self.artistentry.get_text(), self.sourceliststore.get_value(self.sourcecombobox.get_active_iter(), 0), False)

    def song_changed(self):
        if self.get_visible():
            self.updatesongbutton.show()

    def on_updatesongbutton_clicked(self, widget):
        self.set_lyrics(self.remote.info["title"], self.remote.info["artist"], self.key, True)
            
        

if __name__ == "__main__":
    dialog = LyricspanelDialog()
    dialog.show()
    Gtk.main()
