# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
from gi.repository import Gtk
from CoverImage import CoverImage

import logging
logger = logging.getLogger('dockremoteclient')
import gettext
from gettext import gettext as _
gettext.textdomain('dockremoteclient')

class Player():
    def __init__(self, parent):
        self.now_title = "";
        self.now_artist = "";
        self.now_album = "";
        self.playing = 0
        self.handlers = {"sid": self.on_song_changed, "volume": parent.on_volume_changed, "repeat": parent.on_repeat_changed, "shuffle": parent.on_shuffle_changed, "fraction": parent.on_updated_time, "state": self.on_playpause}
        self.queueing = 0

    def set_parent(self, parent):
        self.set_window_title = parent.set_title
        self.remote = parent.remote
        self.notify = parent.notify
        self.soundmenu = parent.soundmenu
        self.queue = parent.queue
        self.timer = parent.timer
        self.playbutton = parent.playbutton
        self.titlelabel = parent.titlelabel
        self.artistlabel = parent.artistlabel
        self.albumlabel = parent.albumlabel
        self.action = None
        self.lyricspaneldialog = parent.lyricspaneldialog
        self.coverimage = CoverImage(self.remote.get_cover, parent.builder, parent.settings, self.update_mpris_cover)
        
    def play_song(self, sid):
        self.remote.play_song(sid)

    def update_mpris_cover(self, arturl):
        self.soundmenu.song_changed(self.remote.info["artist"], self.remote.info["album"], self.remote.info["title"], arturl)
    
    def play_song_from_artist(self, sid, aid):
        self.remote.play_song_from_artist(sid, aid)
    
    def play_song_from_album(self, sid, alid):
        self.remote.play_song_from_album(sid, alid)
    
    def play(self):
        self.action = "play"
        self.playing = 1
        self.remote.play()
        self.timer.play()
        self.playbutton.set_stock_id(Gtk.STOCK_MEDIA_PAUSE)
        self.soundmenu.signal_playing()
        self.update_window_title()

    def pause(self):
        self.action = "pause"
        self.playing = 0
        self.remote.pause()
        self.timer.pause()
        self.playbutton.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
        self.soundmenu.signal_paused()
        self.update_window_title()

    def next(self, queue=1):
        self.action = "next"
        if self.queueing and queue:
            self.next_from_queue()
        else:
            self.remote.next()
        
    def next_from_queue(self):
        logger.debug("Playing first song from the queue")
        d = self.queue.play_first_song()
        if not d:
            self.next(0)

    def prev(self):
        self.action = "prev"
        self.remote.prev()

    def playpause(self, by=""):
        if by == "plaaybutton":
            if self.remote.info["state"] == "playing" and self.playbutton.get_stock_id() == Gtk.STOCK_MEDIA_PAUSE:
                self.pause()
            elif self.remote.info["state"] == "paused" and self.playbutton.get_stock_id() == Gtk.STOCK_MEDIA_PLAY:
                self.play()
            elif self.remote.info["state"] == "playing":
                self.playbutton.set_stock_id(Gtk.STOCK_MEDIA_PAUSE)
                self.soundmenu.signal_playing()
                self.timer.play()
                self.playing = 1
            else:
                self.playbutton.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
                self.soundmenu.signal_paused()
                self.timer.pause()
                self.playing = 0
        else:
            if self.remote.info["state"] == "playing":
                self.pause()
            elif self.remote.info["state"] == "paused" or 1:
                self.play()

    def on_song_changed(self, sid):
        if self.action == None:
            if self.remote.info["state"] == "paused":
                self.action = "pause"
            else:
                self.action = "play" 
        self.titlelabel.set_text(self.remote.info["title"])
        self.artistlabel.set_text(self.remote.info["artist"])
        self.albumlabel.set_text(self.remote.info["album"])
        self.notify.update_notify(self.action)
        self.soundmenu.song_changed(self.remote.info["artist"], self.remote.info["album"], self.remote.info["title"])
        self.soundmenu.signal_playing()
        self.coverimage.update()
        self.lyricspaneldialog.song_changed()
        self.update_window_title()

    def update_window_title(self):
        if self.remote.info["state"] == "paused":
            status = _(" (paused)")
        else:
            status = ""
        if self.remote.info["title"] and self.remote.info["artist"]:
            self.set_window_title("Dock Remote Client - " + self.remote.info["title"] + " - " + self.remote.info["artist"] + status)
        elif self.remote.info["title"]:
            self.set_window_title("Dock Remote Client - " + self.remote.info["title"] + status)
        else:
            self.set_window_title("Dock Remote Client")
        
        

    def on_playpause(self, st):
        if (self.playing == 1 and st == "paused") or (self.playing == 0 and st == "playing"):
            self.playpause("plaaybutton")  
        self.notify.update_notify(self.action)
