# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
from gi.repository import Gtk
from gi.repository import Gdk

import gettext
from gettext import gettext as _
gettext.textdomain('dockremoteclient')

class Queue():
    TARGETS = [
    ('MY_TREE_MODEL_ROW', 0, 0),
    ('text/plain', 0, 1),
    ('TEXT', 0, 2),
    ('STRING', 0, 3),
    ]

    TARGETS2 = [
    ('MY_TREE_MODEL_ROW', Gtk.TargetFlags.SAME_WIDGET, 0),
    ('text/plain', Gtk.TargetFlags.SAME_WIDGET, 1),
    ('TEXT', Gtk.TargetFlags.SAME_WIDGET, 2),
    ('STRING', Gtk.TargetFlags.SAME_WIDGET, 3),
    ]

    TARGETS3 = [
    Gtk.TargetEntry.new('MY_TREE_MODEL_ROW', 0, 0),
    Gtk.TargetEntry.new('text/plain', 0, 1),
    Gtk.TargetEntry.new('TEXT', 0, 2),
    Gtk.TargetEntry.new('STRING', 0, 3),
    ]
       
	def __init__(self, builder, remote, library, database):
        self.library = library
        self.database = database
        self.remote = remote
		self.treeview = builder.get_object("queue")
        self.deletefromqueuebutton = builder.get_object("deletefromqueuebutton")
        self.upinqueuebutton = builder.get_object("upinqueuebutton")
        self.downinqueuebutton = builder.get_object("downinqueuebutton")
        self.queuecontextmenu = builder.get_object("queuecontextmenu")
		self.model = Gtk.ListStore(str, str, str, str, str, str)
        

		cell0 = Gtk.CellRendererText ()
		self.column_title = Gtk.TreeViewColumn(_("Title"), cell0, text=0)
		self.column_title.set_expand(True)
		self.column_title.set_resizable(True)
		self.treeview.append_column (self.column_title)

		cell0 = Gtk.CellRendererText ()
		self.column_artist = Gtk.TreeViewColumn(_("Artist"), cell0, text=1)
		self.column_artist.set_expand(True)
		self.treeview.append_column (self.column_artist)

		self.treeview.set_model(self.model)
        self.treeview.connect("row-activated", self.on_row_clicked)
        self.treeview.connect("key-press-event", self.on_key_pressed)

        self.playingqueueframe = builder.get_object("playingqueueframe")
        self.playingqueuescrolledwindow = builder.get_object("playingqueuescrolledwindow")
        self.playingqueueframe.drag_dest_set(Gtk.DestDefaults.ALL, self.TARGETS3, Gdk.DragAction.DEFAULT | Gdk.DragAction.COPY)
        self.playingqueueframe.drag_dest_add_text_targets()
        self.playingqueueframe.connect("drag-data-received", self.drag_data_received_data)
        self.treeview.enable_model_drag_dest(self.TARGETS,  Gdk.DragAction.DEFAULT | Gdk.DragAction.MOVE)
        self.treeview.drag_dest_add_text_targets()
        self.treeview.connect("drag-data-received", self.drag_data_received_data)


        self.treeview.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, self.TARGETS2, Gdk.DragAction.MOVE)
        self.treeview.drag_source_add_text_targets()
        self.treeview.connect("drag_data_get", self.drag_data_get_data)
        self.selection = self.treeview.get_selection()
        self.selection.set_mode(Gtk.SelectionMode.MULTIPLE)

        data = self.database.get_queue()
        if len(data) == 0:
            self.playingqueueframe.show()
            self.playingqueuescrolledwindow.hide()
        else:
            self.playingqueueframe.hide()
            self.playingqueuescrolledwindow.show()
        for i in data:
            self.model.append(i)
        self.deletefromqueuebutton.connect("clicked", self.remove_selected_rows)
        self.downinqueuebutton.connect("clicked", self.move_selected_down)
        self.upinqueuebutton.connect("clicked", self.move_selected_up)
        self.treeview.connect("button-press-event", self.on_queue_button_press_event)
        self.treeview.connect("button-release-event", self.on_queue_button_release_event)
        #self.treeview.connect("leave-notify-event", self.on_queue_button_release_event)
        builder.get_object("deletefromqueueitem").connect("activate", self.remove_selected_rows)
        builder.get_object("playfromqueueitem").connect("activate", self.on_playfromqueueitem_activate)
        self.upinqueueitem = builder.get_object("upinqueueitem")
        self.upinqueueitem.connect("activate", self.move_selected_up)
        self.downinqueueitem = builder.get_object("downinqueueitem")
        self.downinqueueitem.connect("activate", self.move_selected_down)
        self.selection.connect("changed", self.on_selection_changed)
        self.defer_select = False
        self.on_selection_changed(self.selection)

    def returnme(self, selection, model, path, boll, data):
        return data

    def on_key_pressed(self, widget, event):
        if Gdk.keyval_name(event.keyval) == "Delete":
            self.remove_selected_rows(1)

    def on_queue_button_release_event(self, treeview, event):
	    self.selection.set_select_function(self.returnme, True)
        target = self.treeview.get_path_at_pos(int(event.x), int(event.y))	
        if (self.defer_select and target and self.defer_select == target[0] and not (event.x==0 and event.y==0)): # certain drag and drop
            self.treeview.set_cursor(target[0], target[1], False)
        self.defer_select=False

    def on_queue_button_press_event(self, treeview, event):
        if event.button == 3 and self.selection.count_selected_rows():
            self.queuecontextmenu.popup( None, None, None, None, event.button, event.time)
            return True
        target = self.treeview.get_path_at_pos(int(event.x), int(event.y))
        if (target and event.type == Gdk.EventType.BUTTON_PRESS and not (event.state & (Gdk.ModifierType.CONTROL_MASK|Gdk.ModifierType.SHIFT_MASK)) and self.selection.path_is_selected(target[0])):
            self.selection.set_select_function(self.returnme, False)
            self.defer_select = target[0]

    def drag_data_get_data(self, treeview, context, selection, target_id, etime):


        model, pat = self.selection.get_selected_rows()
        iters = []
        data = []
        for i in pat:
            iters.append(self.model.get_iter(i))
        for iter in iters:
            dat = [model.get_value(iter, 0), model.get_value(iter, 1), model.get_value(iter, 2), model.get_value(iter, 3), model.get_value(iter, 4), model.get_value(iter, 5)]
            data.append("||".join(dat))
            self.model.remove(iter)
        selection.set(selection.get_target(), 8, "|**|".join(data))

    def drag_data_received_data(self, treeview, context, x, y, selection, info, etime):
        data = selection.get_data().decode("utf-8").split("|**|")
        songs = []
        for i in data:
            songs.append(i.split("||"))
        drop_info = self.treeview.get_dest_row_at_pos(x, y)
        if drop_info:
            path, position = drop_info
            iter = self.model.get_iter(path)
            if (position == Gtk.TreeViewDropPosition.BEFORE
                or position == Gtk.TreeViewDropPosition.BEFORE):
                self.add_to_queue(songs, iter)
            else:
                self.add_to_queue(songs, self.model.iter_next(iter))
        else:
            self.add_to_queue(songs)
        if context.get_actions() == Gdk.DragAction.MOVE:
            context.finish(True, True, etime)
        return

    def play_first_song(self):
        i = self.queue_isnt_empty()
        if not i:
            return False
        self.remote.play_song(self.model.get_value(i, 3))
        self.remove_from_queue(i)
        return True

    def queue_isnt_empty(self):
        try:
            i = self.model.get_iter(0)
        except:
            return False
        return i
        

    def on_row_clicked(self, treeview, path, col):
        i = self.model.get_iter(path)
        self.remote.play_song(self.model.get_value(i, 3))
        self.play_from_queue(i)

    def on_playfromqueueitem_activate(self, menu):
        pat = self.selection.get_selected_rows()
        i = self.model.get_iter(pat[1][0])
        self.remote.play_song(self.model.get_value(i, 3))
        self.play_from_queue(i)

    def add_to_queue(self, songs, iter=-1):
        self.playingqueueframe.hide()
        self.playingqueuescrolledwindow.show()
        if type(songs[0]) != list:
            songs = [songs]
        for song in songs:
            if iter == -1:
                self.model.append(song)
            else:
                self.model.insert_before(iter, song)
            self.database.add_to_queue(song)
            

    def play_from_queue(self, iter):
        song = self.model.get_value(iter, 3)
        self.remote.play_song(song)
        self.remove_from_queue(iter)

    def remove_from_queue(self, iter):
        song = self.model.get_value(iter, 3)
        self.model.remove(iter)
        self.database.remove_from_queue(song)
        try:
            i = self.model.get_iter(0)
        except:
            self.playingqueueframe.show()
            self.playingqueuescrolledwindow.hide()

    def on_selection_changed(self, selection):
        if not self.selection.count_selected_rows():
            self.deletefromqueuebutton.set_sensitive(False)
            self.downinqueuebutton.set_sensitive(False)
            self.upinqueuebutton.set_sensitive(False)
            return
        pat = self.selection.get_selected_rows()
        iter = self.model.get_iter(pat[1][0])
        path = int(self.model.get_string_from_iter(iter))
        self.deletefromqueuebutton.set_sensitive(True)
        if path == 0:
            self.upinqueuebutton.set_sensitive(False)
            self.upinqueueitem.set_sensitive(False)
        else:
            self.upinqueuebutton.set_sensitive(True)
            self.upinqueueitem.set_sensitive(True)
        if len(self.model) > path + len(pat):
            self.downinqueuebutton.set_sensitive(True)
            self.downinqueueitem.set_sensitive(True)
        else:
            self.downinqueuebutton.set_sensitive(False)
            self.downinqueueitem.set_sensitive(False)

    def remove_selected_rows(self, widget):
        pat = self.selection.get_selected_rows()
        iters = []
        for i in pat[1]:
            iters.append(self.model.get_iter(i))
        for i in iters:
            self.remove_from_queue(i)
        self.on_selection_changed(self.selection)

    def move_selected_down(self, widget):
        pat = self.selection.get_selected_rows()
        it = self.model.get_iter(pat[1][len(pat[1]) -1])
        it = self.model.iter_next(it)
        it2 = self.model.get_iter(pat[1][0])
        self.model.move_before(it, it2)
        self.on_selection_changed(self.selection)

    def move_selected_up(self, widget):
        pat = self.selection.get_selected_rows()
        it = self.model.get_iter(pat[1][0])
        path = int(self.model.get_string_from_iter(it))
        it = self.model.get_iter_from_string(str(path - 1))
        it2 = self.model.get_iter(pat[1][len(pat[1]) -1])
        self.model.move_after(it, it2)
        self.on_selection_changed(self.selection)
        
    def add_songs_to_queue(self, songlist):
        for song in songlist:
            self.model.append(song)
            self.database.add_to_queue(song)
