# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
import urllib
import time
from gi.repository import GObject
import sys

class Remote():
	def __init__(self, url, change_handlers, handle_error):
		self.update_ip_address(url)
		self.info = {"title": '', "artist": '', "album": '', "sid": '', "duration": 0, "state": 'paused', "volume": 0, "repeat": '', "shuffle": '', "fraction": 0, "time": 0}
        self.changed = change_handlers
        self.handle_error = handle_error
        self.init_string = {"title": '"title": "', "artist": '"artist": "', "album": '"albumTitle": "', "sid": '"trackId": "', "duration": '"duration": ', "state": '"playbackState": "', "volume": '"volume": ', "repeat": '"repeatMode": "', "shuffle": '"shuffleMode": "', "fraction": '"timeFraction": '}
        self.end_string = {"title": '",\n"', "artist": '",\n"', "album": '",\n"', "sid": '",\n"', "duration": ',\n"', "state": '",\n"', "volume": ',\n"', "repeat": '",\n"', "shuffle": '",\n"', "fraction": ',\n"'}
	
    def play_song(self, sid):
        self.send_order("playSong?sid=" + str(sid))

    def play_song_from_artist(self, sid, aid):
        self.send_order("playSongForArtist?sid=" + str(sid) + "&aid=" + str(aid))

    def play_song_from_album(self, sid, alid):
        self.send_order("playSongFromAlbum?sid=" + str(sid) + "&alid=" + str(alid))

	def play(self):
		self.send_order("play")

	def toggle_play(self):
        if self.info["state"] == "playing":
		    self.send_order("pause")
        else:
		    self.send_order("play")

    def update(self, is_process=False):
		self.send_order("whatsNew", is_process)
	
	def pause(self):
		self.send_order("pause")
	
	def next(self):
		self.send_order("next")

    def toggle_shuffle(self):
        self.send_order("toggleShuffle")

    def toggle_repeat(self):
        self.send_order("toggleRepeat")
	
	def prev(self):
		self.send_order("prev")

    def set_volume(self, val):
        self.send_order("setVolume?level=" + str(val))

    def set_fraction(self, val):
        self.send_order("setTimeFraction?value=" + str(val))

	def send_order(self, order, is_process=False):
        init = time.time()
        try:
		    stuff = urllib.urlopen(self.url + order).read();
        except Exception as e:
            print self.url + order
            print "Error :", e
            if is_process:
                GObject.idle_add(self.handle_error, e)
            else:
                self.handle_error(e)
            return
		self.update_with_answer(stuff, (time.time() - init)/2)

    def get_value(self, stuff, init, fin='",\n"', reverse=0):
        if reverse:
            ini = stuff.rfind(init) + len(init)
        else:
            ini = stuff.find(init) + len(init)
		return stuff[ini:stuff.find(fin, ini)]

    def update_value(self, stuff, value):
        new = self.get_value(stuff, self.init_string[value], self.end_string[value], value == "duration")
        if self.info[value] != new:
            self.info[value] = new
            if value in "sid volume fraction state shuffle repeat".split():
                return new
        return False

	def update_with_answer(self, stuff, delay):
        self.info["time"] = time.time() - delay
        to_update = {}
        for i in "title artist album volume state shuffle repeat fraction duration sid".split():
            n = self.update_value(stuff, i)
            if n or i == "fraction":
                to_update[i] = n
        for i in to_update:
            self.changed[i](to_update[i])

    def get_cover(self):
        try:
            data = urllib.urlopen(self.url + "nowPlayingImage.jpg").read()
        except Exception as e:
            print "Error :", e
            self.handle_error(e)
            return None
        return data
    def update_ip_address(self, url):
        if url[:7] == "http://":
            urlstart = 7
        elif url[:1] == "/":
            urlstart = 1
        else:
            urlstart = 0
        url = url[urlstart:]
        if url[-2:] == "//":
            endurl = -2
        elif url[-1:] == "/":
            endurl = -1
        else:
            endurl = len(url)
        url = url[:endurl]
		self.url = "http://" + url + "/"
        
